-- File name   : EFX_CLOCK.MS
-- Title       : Clock Animator
-- Category    : Controller
-- Environment : 3ds max 4
-- Description : euler XYZ rotation controller experiment
-- Reference   : CLOCK.MAX

-- Copyright (c) 1998-2001, EffectWare, Moon-Sun Choi
-- http://www.effectware.com
-- aeroplus@effectware.com

-- general functions ----------------------------------------------------------

-- frame : current frame
-- node : rotation node
-- angle : angle (in radian)

fn set_time frame node angle =
(
	local nc, rc, k

	nc = node.rotation.controller
	rc = nc.y_rotation.controller
	k = addNewKey rc frame
	k.value = angle -- degToRad (angle) : fixed for 4.0
)

-- frame : current time
-- hnode : hour hand
-- mnode : min hand
-- snode : sec hand
-- mode : analog (= false) or digital (= true)

fn set_clock frame hnode mnode snode sec mode =
(
	local h, m, s

	-- 360 / 60 = 6
	sec *= 6.0

	-- 12 * 60 = 720
	h = sec / 720.0
	m = sec / 60.0

	-- for quartz movement
	if mode == true then 
		s = floor (sec / 6.0) * 6.0
	else
		s = sec

	if hnode != undefined then set_time frame hnode h
	if mnode != undefined then set_time frame mnode m
	if snode != undefined then set_time frame snode s
)

-- main program ---------------------------------------------------------------

rollout efx_rollout_clock_about "About"
(
	label l1 "Clock Animator"
	label l2 "Version 1.0"
	label l3 "Choi, Moon-Sun"
	label l4 "Copyright (c) 1998-2001"
	label l5 "http://www.effectware.com"
	label l6 "aeroplus@effectware.com"
)

rollout efx_rollout_clock_param "Parameters"
(
	local hnode, mnode, snode, start, end, sec

	group "Node"
	(
		pickbutton pick_hour "  Pick Hour  "
		label hour_name
		pickbutton pick_min "  Pick Min  "
		label min_name
		pickbutton pick_sec "  Pick Sec  "
		label sec_name
	)

	spinner shour "Hour:" type:#integer range:[0, 23, 0]
	spinner smin "Minute:" type:#integer range:[0, 59, 0]
	spinner ssec "Second:" type:#integer range:[0, 59, 0]
	spinner step "FPS:" range:[0, 1000000, 30]
	checkbox quartz "using quartz movement" checked:false
	checkbox reverse "reverse animation" checked:false

	button create_key "  Create Key  "
	label status

	-- constructor
	on clock_animator open do
	(
		hour_name.text = "None"
		min_name.text = "None"
		sec_name.text = "None"
	)

	-- select hour hand
	on pick_hour picked obj do
	(
		hnode = obj
		hnode.rotation.controller = Euler_XYZ ()
		hour_name.text = hnode.name
	)

	-- select minute hand
	on pick_min picked obj do
	(
		mnode = obj
		mnode.rotation.controller = Euler_XYZ ()
		min_name.text = mnode.name
	)

	-- select second hand
	on pick_sec picked obj do
	(
		snode = obj
		snode.rotation.controller = Euler_XYZ ()
		sec_name.text = snode.name
	)

	-- create key frame
	on create_key pressed do
	(
		start = animationRange.start.frame
		end = animationRange.end.frame

		-- time in second
		sec = ssec.value + (smin.value + (shour.value * 60.0)) * 60.0

		-- animation start
		undo on
		(
			animate on for t in start to end do at time t
			(
				status.text = "Processing frame " + (t as string)
				set_clock t hnode mnode snode sec quartz.checked

				-- increase second
				if reverse.checked then
					sec -= 1.0 / step.value
				else
					sec += 1.0 / step.value
			)
		)
		status.text = ""
	)
)

-- floater routines -----------------------------------------------------------

-- global variables

global efx_floater_clock
global efx_rollout_clock_about
global efx_rollout_clock_param

-- close old floater
if efx_floater_clock != undefined then closeRolloutFloater efx_floater_clock
-- add new floater
efx_floater_clock = newRolloutFloater "Clock Animator" 200 535
-- add rollout
addRollout efx_rollout_clock_about efx_floater_clock
addRollout efx_rollout_clock_param efx_floater_clock
